{********************************************************************}
{        MiscUtil Unit. Lots of useful functions/procedures          }
{        Copyright (C) 1996-1997 by Artem A. Berman                  }
{          N o t   f o r   d i s t r i b u t i n g.                  }
{                                                                    }
{********************************************************************}

unit Miscutil;

interface

uses Windows, LZExpand, Classes, SysUtils, Forms, Graphics;



{ Next five are copyright (c) by 
  Stefan Boether <100023.275@compuserve.com> }

function StrToColor(const Item: string): TColor;
function ColorToStr(Item: TColor): string;
function StrToFont(const Item: string): TFont;
function FontToStr(Value: TFont): string;
function StrToken(var s: string; Seperator: Char): string;

{ Mines from here }

function SetActiveApp(FCaption: PChar): Boolean;
function ExtractFileDriveString(FPath: TFileName): TFileName;
function ExtractFileDriveChar(FPath: TFileName): Char;
function JustFileName(FName: TFileName): TFileName;
function BuildDirectoryFileList(FDirectory, Filter: string; FOnlyNames: Boolean): TStringList;
function BuildFullFileName(FName: TFileName): TFileName;

function Empty(ss: string): Boolean;
function PosFromPos(SubStr, MainStr: string; Posit: Integer): Integer;
function CodeString(WhatArray: string; Sym: Byte): string;
function FileLZCopy(fSource, fDest: PChar): Boolean;
function WCharToString(AValue: Array of WCHAR): string;

implementation


function SetActiveApp(FCaption: PChar): Boolean;
var PWnd: HWnd;
begin
  Result := False;
  PWnd := FindWindow(nil, FCaption);
  if PWnd <> 0 then
  begin
    SetActiveWindow(PWnd);
    Result := True;
  end;
end;

function  StrToColor(const Item: string): TColor;
begin Result := TColor(StrToInt(Item)); end;

function ColorToStr(Item: TColor): string;
begin Result := IntToStr(Longint(Item)); end;

function StrToFont(const Item: string): TFont;
var
  S      : string;
  aStyle : TFontStyles;
begin
  Result := TFont.Create;
  S := Item;
  if not Empty(S) then
    begin
      Result.Name := StrToken(S, ',');
      Result.Size := StrToInt(StrToken(S, ','));
      aStyle := [];
      if Boolean(StrToInt(StrToken(S, ','))) then Include(aStyle, fsBold);
      if Boolean(StrToInt(StrToken(S, ','))) then Include(aStyle, fsItalic);
      if Boolean(StrToInt(StrToken(S, ','))) then Include(aStyle, fsUnderline);
      if Boolean(StrToInt(StrToken(S, ','))) then Include(aStyle, fsStrikeOut);
      Result.Style := aStyle;
      Result.Color := StrToInt(strToken(S, ','));
    end;
end;

function FontToStr(Value: TFont): string;
var S : string;
begin
  S := Value.Name + ',';
  S := S + IntToStr(Value.Size) + ',';
  S := S + IntToStr(Ord(fsBold in Value.Style)) + ',';
  S := S + IntToStr(Ord(fsItalic in Value.Style)) + ',';
  S := S + IntToStr(Ord(fsUnderline in Value.Style)) + ',';
  S := S + IntToStr(Ord(fsStrikeOut in Value.Style)) + ',';
  S := S + IntToStr(Value.Color);
  Result := S;
end;

function ExtractFileDriveString(FPath: TFileName): TFileName;
begin Result := Copy(FPath, 1, Pos('\', FPath)); end;

function ExtractFileDriveChar(FPath: TFileName): Char;
begin Result := FPath[1]; end;

function BuildDirectoryFileList(FDirectory, Filter: string; FOnlyNames: Boolean): TStringList;
var
  handFile: Integer;
  ARec: TSearchRec;
  fName: TFileName;
begin
  FDirectory := FDirectory + Filter;
  Result := TStringList.Create;
  handFile := FindFirst(FDirectory, faArchive, ARec);
  if handFile <> 0 then Exit else
  while True do
  begin
    if (ARec.Attr and faArchive) = faArchive then
    begin
      fName := ARec.Name;
      if FOnlyNames then fName := JustFileName(fName);
      Result.Add(fName);
    end;
    handFile := FindNext(ARec);
    if handFile <> 0 then break;
  end;
  FindClose(ARec);  
end;

function JustFileName(FName: TFileName): TFileName;
begin
  Result := Copy(FName, 0, Pos(ExtractFileExt(Fname), FName) - 1);
end;

function BuildFullFileName(FName: TFileName): TFileName;
begin Result := ExtractFilePath(ParamStr(0)) + FName; end;

function Empty(ss: string): Boolean;
begin if ss = '' then Result := True else Result := False; end;

function PosFromPos(SubStr, MainStr: string; Posit: Integer): Integer;
begin
  MainStr := Copy(MainStr, Posit, Length(MainStr));
  Result := Pos(SubStr, MainStr);
end;

function CodeString(WhatArray: string; Sym: Byte): string;
var Index : Word;
begin
{$IFDEF WIN32}
   exit;
{$ELSE}
  for Index := 1 to High(WhatArray) do
  WhatArray[Index] := Chr(Ord(WhatArray[Index]) XOR Sym);
  Result := WhatArray;
{$ENDIF}
end;

function StrToken(var s: string; Seperator: Char): string;
var
  I               : Word;
  Item            : String;
begin
  Item := '';
  I := Pos(Seperator, s);
  if I <> 0 then
  begin
    Item := Copy(s, 1, I-1);
    Delete(s, 1, I);
  end else
  begin
    Item := s;
    s := '';
  end;
  Result := Item;
end;

function FileLZCopy(fSource, fDest: PChar): Boolean;
var
  Source, Dest: Integer;
  Struct: TOFSTRUCT;
begin
  Source := LZOpenFile(fSource, Struct, OF_READ);
  Dest := LZOpenFile(fDest, Struct, OF_CREATE);
  Result := LZCopy(Source, Dest) > 0;
  LZClose(Source);
  LZClose(Dest);
end;

function WCharToString(AValue: Array of WCHAR): string;
var 
  Index: Integer;
  CH: Char;
begin
  for Index := 0 to High(AValue) do 
  begin
    CH := Char(AValue[Index]);
    Result := Result + CH;
  end;
end;

end.
