{***********************************************************************}
{                                                                       }
{           Copyright (C) 1996 - 1997 by Artem A. Berman                }
{           E-mail: art@aber.kherson.ua                                 }
{                                                                       }
{***********************************************************************}
{ This components can be freely used and distributed in commercial and  }
{ private environments, provied this notice is not modified in any way. }
{ ----------------------------------------------------------------------}
{ Version 1.0.  Date last modified:  05/10/96                           }
{ Version 1.01  Date last modified:  16-Mar-97                          }
{    + Fixed bug with Sorted property, now it works in the correct way. }
{    + Fixed bug when error occuired when you tried to press enter and  }
{      no one Item has been selected                                    }
{    + Under Win 32 names of folders/files are case sensitive now       }
{    + StringCase property - case of the stings in the TExplorerBox     }
{    + Builds directory structure faster                                }
{ ----------------------------------------------------------------------}

unit Explorer;

interface

{$IFDEF WIN32}
  uses
    SysUtils, Windows, Messages, Classes, Graphics, Controls,
    Forms, Dialogs, StdCtrls, Menus;
{$ELSE}
  uses
    SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
    Forms, Dialogs, StdCtrls, Menus;
{$ENDIF}

type
  TStringsCase = (cs_Normal, cs_Upper, cs_Lower);

  TExplorerBox = class(TCustomListBox)
  private
    FFileName,
    FDirectory: TFileName;
    FBitmap: TBitmap;
    FOnFileChange,
    FOnDirChange: TNotifyEvent;
    FFilter: string;
    FStringsCase: TStringsCase;
    procedure SetDirectory(Value: TFileName);
    procedure SetStringsCase(ACase: TStringsCase);
  protected
    procedure DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
    procedure Update; virtual;
    procedure Click; override;
    procedure DblClick; override;
    procedure RePaint; override;
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure CreateWnd; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function RPos(const SubStr: Char; Str: string): Word;
  published
    property Directory: TFileName read FDirectory write SetDirectory;
    property FileName: TFileName read FFileName;
    property Filter: string read FFilter write FFilter;
    property StringsCase: TStringsCase read FStringsCase write SetStringsCase;
    property Align;
    property BorderStyle;
    property Columns;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property IntegralHeight;
    property ItemHeight;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Sorted;
    property Style;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnDragDrop;
    property OnDragOver;
    property OnDrawItem;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnMeasureItem;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnDblClick;
    property OnFileChange: TNotifyEvent read FOnFileChange write FOnFileChange;
    property OnDirChange: TNotifyEvent read FOnDirChange write FOnDirChange;
  end;

procedure Register;

implementation

{$R RESOURCE}

constructor TExplorerBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Style := lbOwnerDrawFixed;
  Columns := 4;
  FFilter := '*.*';
  FStringsCase := cs_Normal;
  FBitmap := TBitmap.Create;
  FBitmap.Handle := LoadBitmap(HInstance, 'BITMAP_1');
end;

destructor TExplorerBox.Destroy;
begin
  FBitmap.Free;
  inherited Destroy;
end;

function TExplorerBox.RPos(const SubStr: Char; Str: string): Word;
var
  i: Word;
begin
  Result := 0;
  repeat
    if Str[Length(Str)] = SubStr then exit else
    begin
      SetLength(Str, Length(Str) - 1);
      Inc(Result)
    end
  until Length(Str) = 0;
end;

procedure TExplorerBox.SetDirectory(Value: TFileName);
begin
  if Value[Length(Value)] <> '\' then
    FDirectory := Value + '\'
  else
    FDirectory := Value;
  Update;
end;

procedure TExplorerBox.Update;
var
  ARec: TSearchRec;
  Res, Index: Integer;
  SList: TStringList;
begin
  Clear;
  SList := TStringList.Create;
  SList.Sorted := Sorted;

  Res := FindFirst(FDirectory + Filter, faArchive or faDirectory, ARec);

  while Res = 0 do
  begin
      case FStringsCase of
        cs_Upper: ARec.Name := UpperCase(ARec.Name);
        cs_Lower: ARec.Name := LowerCase(ARec.Name);
      end;

      if (ARec.Attr and faDirectory) > 0 then
              if ARec.Name <> '.' then Items.Add(ARec.Name);
      
      if (ARec.Attr and faArchive) > 0 then
              SList.Add(ARec.Name);

      Res := FindNext(ARec);
  end;

  for Index := 0 to SList.Count - 1 do
        Items.Insert(Items.Count, SList.Strings[Index]);

  SList.Free;
end;

procedure TExplorerBox.Click;
var Res: Integer;
    AFull: string;
begin
  inherited Click;
  AFull := FDirectory + Items.Strings[ItemIndex];
  Res := FileGetAttr(AFull);
  if Res = faArchive then
  begin
    if Assigned(FOnFileChange) then FOnFileChange(Self);
    FFileName := FDirectory + Items.Strings[ItemIndex]
  end
end;

procedure TExplorerBox.DblClick;
var Res: Integer;
    DirOld, AFull: string;
begin
  if ItemIndex = -1 then Exit;
  if Items.Strings[ItemIndex] = '..' then
  begin
    SetLength(FDirectory, Length(FDirectory) - 1);
    AFull := Copy(FDirectory, 0, Length(FDirectory) - RPos('\', FDirectory));
    DirOld := Copy(FDirectory, Length(FDirectory) - RPos('\', FDirectory) + 1,
                   Length(FDirectory));
    Directory := AFull;
    if Assigned(FOnDirChange) then FOnDirChange(Self);
    ItemIndex := Items.IndexOf(DirOld);
    exit
  end;
  AFull := FDirectory + Items.Strings[ItemIndex];
  Res := FileGetAttr(AFull);
  if (Res AND faDirectory) > 0 then
  begin
    Directory := AFull;
    if Assigned(FOnDirChange) then FOnDirChange(Self);
    ItemIndex := 0
  end else inherited DblClick
end;

procedure TExplorerBox.KeyDown(var Key: Word; Shift: TShiftState);
begin
  if Key = 13 then DblClick;
end;

procedure TExplorerBox.RePaint;
begin
  SetDirectory(FDirectory);
  inherited RePaint
end;

procedure TExplorerBox.DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  Res: Integer;
begin
  with Canvas do
  begin
    FillRect(Rect);
    Res := FileGetAttr(FDirectory + Items.Strings[Index]);
    if ((Res AND faDirectory) < 0) OR (Items.Strings[Index] = '..') then
    begin
      Inherited DrawItem(Index, Rect, State);
      exit
    end else
    begin
      Draw(Rect.Left, Rect.Top, FBitmap);
      TextOut(Rect.Left + FBitmap.Width + 2, Rect.Top, Items[Index]);
    end
  end
end;

procedure TExplorerBox.CreateWnd;
var
  s: string;
begin
  inherited CreateWnd;
  if Directory = '' then
  begin
    GetDir(0, s);
    Directory := s
  end;
end;

procedure TExplorerBox.SetStringsCase(ACase: TStringsCase);
begin
  FStringsCase := ACase;
  Update;
end;

procedure Register;
begin
  RegisterComponents('Custom', [TExplorerBox]);
end;

end.
